package com.aitrainer.api.test

import com.aitrainer.api.controller.CustomerController
import com.aitrainer.api.model.Customer
import com.aitrainer.api.model.User
import com.aitrainer.api.repository.CustomerRepository
import com.aitrainer.api.service.ServiceBeans
import org.junit.jupiter.api.BeforeAll
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.TestInstance
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.http.HttpStatus
import org.springframework.http.ResponseEntity
import kotlin.test.assertEquals
import kotlin.test.assertNotNull

@SpringBootTest
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
class CustomerTests {

    @Autowired
    private var serviceBean: ServiceBeans? = null

    @BeforeAll
    fun init() {
        if ( serviceBean == null ) { serviceBean = ServiceBeans() }
    }

    @Autowired
    private lateinit var customerRepository: CustomerRepository
    private var insertedId: Long? = null

    @Test
    fun testGet() {
        val id: Long = 7
        val customer: Customer = customerRepository.findById( id ).orElse(null)
        assertEquals( customer.name, "Átlag 18 éves fiú")
    }

    @Test
    fun testInsert() {
        val newCustomer = Customer("Bossanyi", "Tibor", "", 48, "m")
        val savedCustomer: Customer = customerRepository.save(newCustomer)
        assertEquals(savedCustomer.age, 48)

        this.insertedId = savedCustomer.customerId

        val customer: Customer = customerRepository.findById( savedCustomer.customerId ).orElse(null)
        assertEquals( customer.firstname, "Tibor")
        
        this.testUpdate(savedCustomer.customerId)
    }

    fun testUpdate( customerId: Long ) {
        val id: Long? = customerId
        assertNotNull(id)

        val updatedCustomer: Customer = customerRepository.findById( id ).orElse(null)

        assertNotNull(updatedCustomer)
        updatedCustomer.firstname ="Tiborka"

        val customer: Customer = customerRepository.save( updatedCustomer )
        assertEquals( customer.firstname, "Tiborka")

        customerRepository.delete(updatedCustomer)
    }

    @Test
    fun testUpdateCustomer() {
        var id:Long = 90

        val customer: Customer = customerRepository.findById( id ).orElse(null)

        customer.customerId = id
        customer.firstname = "Tib"
        customer.name = "Bossi"
        customer.admin = 1
        customer.active = "Y"
        customer.sex = "m"

        customer.birthYear = 1972
        customer.bodyType = "endomorph"
        customer.dataPolicyAllowed = 1
        customer.fitnessLevel = "advanced"

        val updatedCustomer = customerRepository.save(customer)
        assertEquals(updatedCustomer.customerId, 90)
        assertEquals(updatedCustomer.birthYear, 1972)
        assertEquals(updatedCustomer.fitnessLevel, "advanced")
    }

    @Test
    fun testRegistration() {
        val json = "{\"username\":\"bosi@example.com\",\"password\":\"94385\"}"
        val user: User = User().fromJson(json)
        assertEquals(user.username, "bosi@example.com")
        val customer = Customer()
        with(customer) {
            email = user.username
            password = user.password
        }
        val customerController = CustomerController(customerRepository)
        customerController.serviceBeans = serviceBean
        var response: ResponseEntity<*> = customerController.registration(json)
        val newCustomer: Customer? = response.body as Customer
        assertEquals(response.statusCode, HttpStatus.OK)

        val json2 = "{\"username\":\"bosi@example.com\",\"password\":\"934345\"}"
        response = customerController.registration(json2)
        assertEquals(response.statusCode, HttpStatus.BAD_REQUEST)

        if ( newCustomer != null) {
            customerRepository.delete(newCustomer)
        }
    }

    @Test fun testLogin() {
        val json = "{\"username\":\"bosi2@example.com\",\"password\":\"94333385\"}"
        val user: User = User().fromJson(json)
        val customer = Customer()
        with(customer) {
            email = user.username
            password = user.password
        }
        val customerController = CustomerController(customerRepository)
        customerController.serviceBeans = serviceBean
        var response: ResponseEntity<*> = customerController.registration(json)
        val newCustomer: Customer? = response.body as Customer
        assertEquals(response.statusCode, HttpStatus.OK)

        response = customerController.login(json)
        val loginedCustomer: Customer? = response.body as Customer
        assertEquals(response.statusCode, HttpStatus.OK)
        if ( loginedCustomer != null ) {
            assertEquals(loginedCustomer.email, ("bosi2@example.com") )
        } else {
            assert(true)
        }

        if ( newCustomer != null) {
            customerRepository.delete(newCustomer)
        }
    }
}